// constants 
// -------------------------------------------

// definition du cout

E=[5.9623,6.998,8.4363,9.9111,11.018,12.126,13.233,14.541,15.848,17.156,18.463,19.771];

alpha=1000; // cost of backstop technology
nu=3;
theta=0.0000526; 
M0=360;     // initial concentration 
Minf=274;   // concentration preindustrielle 
rho=0.05;   // actualisation cost 
sigma=0.01; // taux de disparition du CO2 dans les puits 
delta=10;   // step size 
beta=0.38;  // taux d'absorption des emissions dans l'atmosphere
e=0.0001
Gamma=0.005;

deff('y=Lambda(t)','y=0.25+0.75*exp(-0.01*delta*t)');
deff( 'y=gama(v)','y=1/(2*Gamma*delta)*(v+sqrt(v*v+e))+1');
deff( 'y=gama_v(v)','y=1/(2*Gamma*delta)*(1+v/sqrt(v*v+e))');

deff('y=psi(u1,u2)','y=u2-u1-delta*Gamma');
deff('y=psi_u1(u1,u2)','y=-1')
deff('y=psi_u2(u1,u2)','y=1')

// cost function and related  derivatives 
// ---------------------------------------

deff('y=L(a,x,t)','y=(alpha*E(t+1)*a^nu*Lambda(t)*gama(psi(x(1),a)))/(1+rho)^(delta*t)');

deff('y=L_cmd(a,x,t)','y=alpha*E(t+1)*(nu*a^(nu-1)*Lambda(t)*gama(psi(x(1),a))+a^nu*Lambda(t)*gama_v(psi(x(1),a)))/(1+rho)^(delta*t)');

deff('y=L_etat(a,x,t)','y(1)=-alpha*E(t+1)*a^nu*Lambda(t)*gama_v(psi(x(1),a))/(1+rho)^(delta*t),y(2)=0');

// system dynamics and related derivatives 
// ---------------------------------------

deff('y=f(a,x,t)','y(1)=a,y(2)=x(2)+10*(beta*E(t+1)*(1-a)-sigma*(x(2)-Minf))');

// Warning f_etat must return the matrix M such that 
// M(i,j) = dfi/dxj 
 
deff('y=f_etat(a,x,t)','y=zeros(2,2),y(1,1)=0,y(2,1)=0,y(1,2)=0,y(2,2)=1-sigma*delta');

deff('y=f_cmd(a,x,t)','y(1)=1,y(2)=-beta*E(t+1)*delta');

// final cost and its derivatives 
// ---------------------------------------

deff('y=phi(x,t)','y=0');

deff('y=phi_etat(x,t)','y(1)=0,y(2)=0');

// initial state x 
x0=[0;360];

// bounds and initial value on the control  a
T=12 
ab=zeros(1,T);
ah=zeros(1,T)+1;
a0=rand(1,T);

// state constraints 
xinf=[0;0]*ones(1,T+1);
xsup=[1;450]*ones(1,T+1);
xinf(2,T+1)=390;
xsup(2,T+1)=390;


nom_L=["L","L_cmd","L_etat"];
nom_f=["f","f_cmd","f_etat"];
nom_phi=["phi","phi_etat"];


[Uopt,Eopt,Jopt,Gradopt]=dynoptimsc(nom_L,nom_f,nom_phi,ab,a0,ah,xinf,x0,xsup,'lag',k=zeros(2,1)+1,150,40,'ar',200,200,0.000E+00,0.4441E-15,0*a0+0.000E+00,'algo','qn');


years=1990+10*(0:T+1);
xset('window',0);
xbasc()
plot2d2(years(1:T),Uopt,2) 
xstring(2010,0.5,"time evolution of control")

Emax=xsup(2,1)*ones(1,T+2);
Efinal=xsup(2,T+1)*ones(1,T+2);
Eres=zeros(T+2);
Eres(1:T+1)=Eopt(2,:);
Eres(T+2)=Eres(T+1);

xset('window',1);
xbasc() 
plot2d2(years,Eres(1:T+2),5) 
plot2d2(years,Emax(1:T+2),6)
plot2d2(years,Efinal(1:T+2),7)

xstring(2030,390,"time evolution of second component of state" )














