function [Uopt,Xopt,critopt,gradopt]=dynoptimsc(name_L,name_f,name_phi,U_low,U_init,U_high,X_low,X_init,X_high,varargin)
//------------------------------------------------------------------
//                       optimisation dynamique
//  version  
//          avec contraintes de bornes sur l'etat
//          avec primitives+
//------------------------------------------------------------------
//nom_... sont les noms des fonctions utiliser par le probleme qui consiste
// a trouver min J(u) avec J(u)=somme( L(u[t],x[t],t))+phi(u[T],x[T],T)
//           0<u<1               t=0..T-1
// en tenant compte de la dynamique :  x[t+1]=f(u[t],x[t],t)
//-----------------------------------------------------------------------------
//-------------------- on fixe les fonctions a utiliser---------------------
//ATTENTION l'odre est le suivant :

doc_setf(name_L(1),name_L(2),name_L(3),name_f(1),name_f(2),name_f(3),name_phi(1),name_phi(2));

doc_setf_sc(name_L(1),name_L(2),name_L(3),name_f(1),name_f(2),name_f(3),name_phi(1),name_phi(2));

//------------------ definition des parametres par defaut  ------------------
//les tailles des variables d'etat et de commandes
s_c=size(U_init)
s_e=size(X_init)
dynsc_pas=s_c(2)
n_e=s_e(1)

// les parametres d'optim

nap = 200;
dynsc_iter= 200;
epsf= 0.000E+00;
epsg= 0.4441E-15;
epsx= 0*U_init+0.0000E+00;

// les parametres du lagrangien

dynsc_k=zeros(n_e,1)+1;
dynsc_p=zeros(2*n_e*(dynsc_pas+1),1);
dynsc_c=100;
dynsc_r=50;

//les parametres d'Uzawa
maxiter=10;
stop=0.0000000001;


//algorithme utilise par optim
algo='qn';

//------------------ recuperation des parametres optionnels -----------------

[lhs,rhs]=argn(0)


deff('b=is_dynsc_key(mot)','b=...
    (mot==''ar'')|(mot==''lag'')|...
    (mot==''uzawa'')|(mot==''algo'')|(mot==''info'')');


// verification  des arguments optionnels

if rhs < 9 then
  error("at least one argument is missing",1001)
elseif rhs >9 then
  for i=1:rhs-9
    if (varargin(i) == "ar") then
      if rhs<9+i+5 then
	error("after ar 5 arguments expected",1001)
      else
	for j=1:5
	  if is_dynsc_key(varargin(i+j)) then
	    error("after ar 5 arguments expected",1001)
	  end
	end
	i=i+5
      end
    elseif (varargin(i) == "lag") then
      if rhs<9+i+3 then
	error("after lag 3 arguments expected",1001)
      else
	for j=1:3
	  if is_dynsc_key(varargin(i+j)) then
	    error("after lag 3 arguments expected",1001)
	  end
	end
	i=i+3
      end
    elseif (varargin(i) == "uzawa") then
      if rhs<9+i+2 then
	error("after uzawa 2 arguments expected",1001)
      else
	for j=1:2
	  if is_dynsc_key(varargin(i+j)) then
	    error("after uzawa 2 arguments expected",1001)
	  end
	end
	i=i+2
      end
    elseif (varargin(i) == "algo") then
      if rhs<9+i+1 then
	error("after algo 2 arguments expected",1001)
      else
	if is_dynsc_key(varargin(i+1)) then
	  error("after algo 2 arguments expected",1001)
	end
	i=i+1
      end
    end
  end
end    

//reccuperation des arguments optionnels

for i=1:rhs-9
  if (varargin(i) == "ar") then
    nap=varargin(i+1);
    dynsc_iter=varargin(i+2);
    epsf=varargin(i+3);
    epsg=varargin(i+4);
    epsx=varargin(i+5);
    i=i+5;
  elseif varargin(i) == "lag"
    dynsc_k=varargin(i+1);
    dynsc_c=varargin(i+2);
    dynsc_r=varargin(i+3);
    i=i+3;
  elseif varargin(i) == "uzawa"
    maxiter=varargin(i+1);
    stop=varargin(i+2);
    i=i+2;
  elseif varargin(i) == "algo"
    algo=varargin(i+1);
    i=i+1;
  end
end

//------------------------gestion de la contraintes sur l'etat -----------

//on regarde si les contraintes sur l'etat sont definies 
//             pour chaque pas de temps

s_low=size(X_low);
t_low=s_low(2);

if t_low==1 then
	X_low=X_low*ones(1,dynsc_pas+1);
end;


s_high=size(X_high);
t_high=s_high(2);

if t_high==1 then
	X_high=X_high*ones(1,dynsc_pas+1);
end;

//---------------------------- optimisation ---------------------------------
// fonction de minimisation de la fonction J situee
// dans le fichier cout_dyn.sce et programmee en c 
//(fonctions.c,cout_dyn.c,interface.c)


xstart=U_init;

for i=1:maxiter 
	// une optimsation du primal
	algo="qn"
	[fopt,xopt,gopt]=optim(dyn_ora_sc,'b',U_low,U_high,xstart,algo,'ar',nap,dynsc_iter,epsg,epsf,epsx,imp=2);
	// un pas de dual

	dynsc_t=doc_T_sc(xopt,X_init,X_low,X_high,dynsc_k);
	dynsc_p=(1-dynsc_r/dynsc_c)*dynsc_p+dynsc_r/dynsc_c*maxi(0,dynsc_p+dynsc_c*dynsc_t);

	// test d'arret
//pause
	if i>1 then
		if norm(xopt-xsav)<stop then break end;
	end;

	fsav=fopt;
	xsav=xopt;
	xstart=xopt;
write(%io(2),i);
end;

// les resultats si arret
Uopt=xopt;
[critopt,gradopt,Xopt]=doc_J(Uopt,X_init);
 
//--------------------------------------------------------------------------
// une fonction qui fait l'intermediaire entre J et optim
//  raison de compatibilite d' arguments et de retour

function [fo,go,ind]=dyn_ora_sc(x,ind)
[fo,go,lo]=doc_J_sc(x,X_init,X_low,X_high,dynsc_k,dynsc_c,dynsc_p);













