.TH dynoptimsc 1 "July 2000" "Scilab Group" "Scilab Function"
.so ../sci.an
.SH NAME
dynoptimsc - the discret optimal command tool with state contraints
.SH CALLING SEQUENCE
.nf
[uopt,Xopt,critopt,gradopt]=dynoptim(L,f,phi,ub,u0,uh,Xinf,X0,Xsup,[stop],[lag],[uzawa],[algo])
.fi

.SH PARAMETERS
.IP L,...,phi_sta
string of characters, name of scilab macros (see below)
.IP ub,uh
scilab vectors which size is the size of the
command u. ub is the lower bound fand uh the upper one.
The function is expecting that size(u0)=[n_u,step], n_u the
dimension of the command, and step the number 
of step in the dynamic. 
.IP u0	
The initial value of the variable u to be minimized
.IP X0
The initial state value, the function is expecting that
size(X0)=[n_X,1], with n_x the dimension of the state.
.IP Xinf,Xsup
the state upper and lower bounds
.IP stop
sequence of optional parameters controlling the convergence of the
algorithm
.nf
stop=  'ar',nap, iter ,epsg ,epsf ,epsx
.fi
.IP algo
specify the algorithm to be used
.nf
algo ='algo',('gc'or 'qn')
.fi

As the dynoptim routine use the primitive optim, theses sequences is
exactly tne same that the stop and algo sequences used for optim.

.IP lag
parameter of the lagrangian cost function used to incorporate the
state bounds contraints
.nf
lag= 'lag',ctr_norm,c,rho
.fi
ctr_norm is a row of real numbers scaling the contraints'vector :
.RS
.nf 
[ctr_norm(1)*X(1)-Xh(1); ...]
.fi
.RE
the size of \fVctr_norm\fR is the size of \fVX0\fR;
\fVc,rho\fR are the well known parameters used in dual lagrangian problems;

 

.IP uzawa
sequence of optional parameters controlling the convergence of the
uzawa algorithm
.nf
uzawa= 'uzawa',maxiter,stop
.fi
\fVmaxiter\fR is the maximal alowed number of dual steps
\fVstop\fR is the threshold of gradient

.SH DESCRIPTION 
Non-linear optimization routine for discret Lagrange problem
without contraints or only with bound contraints imposed to the
command u :
.RS
min J(u) = sum L(u(t),X(t),t) + phi(X(T),T)
 ub<u<uh   t=0..T-1 
.RE
for a control u(t) defined on {0..T-1} the state X of the system is
the solution of the system of equations 
.RS
.fi
X(0)=X0 , X(t+1)=f(u(t),X(t),t)
.nf
.RE


.SH BEWARE
\fVL\fR is a matrix of three strings : name of the cost function, name of
the partial derivative of the cost with
respect to u, and name of the 
partial derivative of the cost with respect to X. The declarations of
\fVf\fR and \fVphi\fR follow the same syntax.

.SH DEFAULT PARAMETERS
.IP stop parameters 
.nf 
nap = 200;
iter= 200;
epsf= 0.000E+00;
epsg= 0.4441E-15;
.fi
epsx= is a vector (or matrix) of zeros, which size is the size of the  initial command \fVu0\fR ;
.IP lag parameters  
k is a vector (or matrix) of ones, which size is the size of the
initial state \fVX0\fR ;
.nf
c=100;
r=50;
.fi
.IP uzawa parameters
.nf
maxiter=10;
stop=0.001;
.fi

.SH EXEMPLE

.nf


///////////  several parameters ///////////////////
 
E=[5.9623,6.998,8.4363,9.9111,11.018,12.126,13.233,14.541,15.848,17.156,18.463,19.771];

alpha=1000; 
nu=3;
theta=0.0000526; 
M0=360; 
Minf=274;   
rho=0.05;   
sigma=0.01; 
delta=10;   
beta=0.38;  
e=0.0001
Gamma=0.001;

deff('y=Lambda(t)','y=0.25+0.75*exp(-0.01*delta*t)');

deff( 'y=gama(v)','y=1/(2*Gamma*delta)*(v+sqrt(v*v+e))+1');
deff( 'y=gama_v(v)','y=1/(2*Gamma*delta)*(1+v/sqrt(v*v+e))');
 
deff('y=psi(u1,u2)','y=u2-u1-delta*Gamma');
deff('y=psi_u1(u1,u2)','y=-1')
deff('y=psi_u2(u1,u2)','y=1')
 
 

//////////The cost and its derivatives///////////////
 
deff('y=L(a,x,t)','y=(alpha*E(t+1)*a^nu*Lambda(t)*gama(psi(x(1),a)))/(1+rho)^(delta*t)');

deff('y=L_cmd(a,x,t)','y=alpha*E(t+1)*(nu*a^(nu-1)*Lambda(t)*gama(psi(x(1),a))+a^nu*Lambda(t)*gama_v(psi(x(1),a)))/(1+rho)^(delta*t)');

deff('y=L_etat(a,x,t)','y(1)=-alpha*E(t+1)*a^nu*Lambda(t)*gama_v(psi(x(1),a))/(1+rho)^(delta*t),y(2)=0');
 

 
//////////The dynamic and its derivatives ///////////////
 

deff('y=f(a,x,t)','y(1)=a,y(2)=x(2)+10*(beta*E(t+1)*(1-a)-sigma*(x(2)-Minf))');
deff('y=f_etat(a,x,t)','y=zeros(2,2),y(1,1)=0,y(2,1)=0,y(1,2)=0,y(2,2)=1-sigma*delta');
deff('y=f_cmd(a,x,t)','y(1)=1,y(2)=-beta*E(t+1)*delta');
 

 
///////// The final cost and its derivatives ///////////
 

deff('y=phi(x,t)','y=0');
deff('y=phi_etat(x,t)','y(1)=0,y(2)=0')
 

 
//////////Paramaters for the optimization/////////////
 

e0=[0;360];
ub=zeros(1,11); //the comand lower bound
uh=zeros(1,11)+1; //the comand upper bound
u0=rand(1,11);
esup=[1;450];//the state upper bound
einf=[0;0]; //the state lower bound
 

//////////The call of the primitive dynoptim /////////
 

L_noms=["L","L_cmd","L_etat"];
f_noms=["f","f_cmd","f_etat"];
phi_noms=["phi","phi_etat"];


[Uopt,Eopt]= dynoptimsc(L_noms,f_noms,phi_noms,ub,u0,uh,einf,e0,esup)

////////// Results /////////////////

T=[0:10];
T=1990+10*T;

xset('window',0);
xbasc(0);
plot2d(T,Uopt,5,"111","command",[1990,0,2100,1],[10,10,10,10]);

xset('window',1);
xbasc(1);
plot2d(T,Eopt(2,1:11),2,"011","state",[1990,360,2100,600],[10,10,10,10]);

.fi 
.SH SEE ALSO
optim - dynoptim
.SH AUTHOR
Sebastien Berthaud, J.Ph.C.












