//****************************************************
//*   Modle de croissance de rfrence              *
//*   politique climatique optimale                  *
//*   F. Lecocq - chp1                               *
//*                                                  *
//****************************************************

// calibrage des parametres pour les emissions

E0=7.1;
PIB0=18;
mu=0.6589861;
nu=-0.00043208;
//annee de depart
T0=1990;

//horizon de projection
T=2290;

//pas de temps
delta=10;

//nombre de periodes
per=(T-T0)/delta;


// parametres du C-cycle

//concentration atmospherique initiale (1990) en ppm
Cini=[750*0.471];
//concentration atmospherique preindustrielle
Minf=274;
//taux d'absorption des emissions dans l'atmosphere en ppm/GtC/an
beta=0.38;
//taux de disparition du CO2 dans les puits en ppm/an
sigma=0.01;


// parametres des fonctions de dommages

//fonctions de dommages lineaires

theta=0.05/(2*Minf-Cini);
deff('y=xsi(m)','y=theta*(m-Cini)');
deff('y=xsi_m(m)','y=theta');



// specifications de la fonction de production

//capital initial (trillion US$90)
K0=95;

//production initiale (trillion US$90)
Y0=18;


//population mondiale initiale (milliard hb)
L0=5.3;
//taux de croissance de la population (population et progres technique)
txcropop= 0.02;
// population active efficace
L=L0*(1+txcropop).^(delta*(0:per));


//coefficient du capital
alpha=(log(Y0/L0))/log(K0/L0);

//fonction de production (Cobb-Douglas)
deff('y=output(K,Lab)','y=K^alpha*Lab^(1-alpha)');
deff('y=output_K(K,Lab)','y=alpha*K^(alpha-1)*Lab^(1-alpha)');



// parametres de la fonction de cout d'abatement (trillion US$90)

//cout de la backstop technology
chi=1; 
// parametre du cout d'abattement
pi=3;	 
// parametre de l'inertie
e=0.0001;
Gamma=0.005;

// fonction de progres technique autonome
deff('y=Lambda(t)','y=0.25+0.75*exp(-0.01*delta*t)');

// lissage de l'inertie de l'abatement
deff( 'y=gama(v)','y=1/(2*Gamma*delta)*(v+sqrt(v*v+e))+1');
deff( 'y=gama_v(v)','y=1/(2*Gamma*delta)*(1+v/sqrt(v*v+e))');
deff('y=psi(u1,u2)','y=u2-u1-delta*Gamma');
deff('y=psi_u1(u1,u2)','y=-1');
deff('y=psi_u2(u1,u2)','y=1');

//ecriture des emissions
deff('y=Em(K,t)','y=E0*((K^alpha)*(L(t+1)^(1-alpha))/PIB0)^(mu+nu*delta*t)');
deff('y=Em_K(K,t)','y=(mu+nu*delta*t)*(alpha*(K^(alpha-1))*(L(t+1)^(1-alpha))/PIB0)*E0*((K^alpha)*(L(t+1)^(1-alpha))/PIB0)^(mu+nu*delta*t-1)');


// autres parametres du modele

// taux de preference pure pour le present
rho=0.05;
// taux de depreciation du capital (%/an)
deprec=0.03;

getf('commandes_a_c.sci','c');

// definition des parametres du probleme

//conditions initiales et bornes sur l'etat et la commande
X0=[95.00;353.25];
Xb=[0;0];
Xh=[1000*K0;1000];


a=[0,0.179746835443038,0.191051995163241,0.208520179372197,0.231632653061224,0.260110294117647,0.293827160493827,...
0.331617647058824,0.375328083989501,0.424473067915691,0.47985347985348,0.540477304632663,0.607292539815591,...
0.678437265214125,0.750252780586451,1,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,1]

I=[4,2.17,2.99,3.76,4.6,5.58,6.76,8.19,9.9,11.98,14.5,17.57,21.36,26.12,32.44,39.93,0,35.28,0,0,0,0,0,0,2.85,2.85,...
2.85,2.85,2.85,2.85,2.85,0]


ub=[14*ones(1,per+1);zeros(1,per+1)];
uh=[100*ones(1,per+1);ones(1,per+1)];
//point de depart de l'algorithme
u0=[I(1:per+1);a(1:per+1)];

Crit_noms=["Crit","Crit_cmd","Crit_etat"];
f_noms=["f","f_cmd","f_etat"];
phi_noms=["phi","phi_etat"];

pause
[Uopt,Xopt,critere,gradient]=dynoptimsc(Crit_noms,f_noms,phi_noms,ub,u0,uh,Xb,X0,Xh);
//[Uopt,Xopt,critere,gradient]=dynoptim(Crit_noms,f_noms,phi_noms,ub,u0,uh,X0);


tps=T0:delta:T;
xset("window",1);
xbasc(1);
plot2d(tps',Uopt(1,:)');
plot2d(tps',I(1:per+1),2);
xstring(2010,0.5,"consommation")

xset("window",2);
xbasc(2);
plot2d(tps',Xopt(1,1:$-1)'); 
xstring(2010,0.5,"capital")

xset("window",3);
xbasc(3);
plot2d(tps',Uopt(2,:)'); 
plot2d(tps',a(1:per+1)',2); 
xstring(2010,0.5,"abattement")

xset("window",4);
xbasc(4);
plot2d(tps',Xopt(2,1:$-1)'); 
xstring(2010,0.5,"concentration")
