#include <stdio.h>
#include <stdlib.h>
#include "utils.h"
#include "fonctions.h"
#include "cout_dyn.h"

/*-------------------------------------------------------------------
 *  compute the cost and its derivative with respect to u 
 * 
 *  J(cmd)= somme  (L(cmd(t),etat(t),t)  + phi(etat(T))
 *        t=0..T-1
 *
 *         etat(0)=e0
 *        etat(t+1)= F(cmd(t),etat(t),t)
 *----------------------------------------------------------------------
 */

extern int BeginVar;

void J_sc_GRAD(double* cmdt, int* n_pas, int* n_cmd, int* n_etat, 
	       double* init_etat, double* etatinf, double* etatsup, 
	       double *alpha, double c, double* p, double* etat, 
	       double* grd, double* FI, int last)
{

  /* les anciennes valeurs des tailles */ 
  static int o_lambda=0,o_gC_cmd=0,o_gf_cmd=0,o_gf_etat=0,o_l=0;
  
  /* les nouvelles */
  int T=*n_pas;  /* nombre de pas de temps */
  int N=*n_etat; /* taille de l'etat       */
  int C=*n_cmd;  /* taille de la commande  */
  
  static double* lambda;
  
  /* des intermediaires */
  double cout; /*pour le calcul du cout*/
  static double* gf_etat=NULL;
  static double* gf_cmd=NULL;
  static double* gC_cmd=NULL;
  static double* l=NULL;
  int i,j,k;
  
  /* si c'est le dernier on nettoie */
  if(last==1)
    {
      net(&lambda);
      net(&gf_etat);
      net(&gf_cmd);
      net(&gC_cmd);
      net(&l);
      return ;
    }

  /* sinon on fait de la place */
  allocdouble(&lambda,N*(T+1),&o_lambda);
  allocdouble(&gf_etat,N*N,&o_gf_etat);
  allocdouble(&gf_cmd,N*C,&o_gf_cmd);
  allocdouble(&gC_cmd,C,&o_gC_cmd);
  allocdouble(&l,N,&o_l);

  /* enfin on dit quelle est la derniere variable Scilab utilisee*/
  BeginVar=11;

  /*-------------------------------DEBUT-------------------------------*
   * on calcul l'etat(N x (T+1)) en fonction des  T commandes   
   *  on initialise l'etat
   */
  
  for(i=0;i<N;i++)
    etat[i]=init_etat[i];
  
  /* on calcule etat(t+1) avec etat(t)*/
  
  for(k=0;k<T;k++)
    f_sc(cmdt+C*k,etat+N*k,&k,etat+(k+1)*N,n_cmd,n_etat);
  
  /* on en profite pour calculer le cout total
   * on initialise les variables de sortie */
  
  *FI=0;
   
  /* calcul de J */
 
  for(k=0;k<T;k++)
    {
      L_sc(cmdt+C*k,etat+N*k,&k,etatinf,etatsup,alpha,c,p,&cout,n_cmd,n_etat);
      *FI +=cout;
    }
  
  phi_sc(etat+N*T,&T,etatinf,etatsup,alpha,c,p,&cout,n_etat);
  *FI+=cout;
    
  /*  on calcul l'etat adjoint (T x N+1)
   *  en fonction des T commandes et des T*(N+1) etats             
   */

  /* on initialise les lambdas */
  
  for(k=0;k<T+1;k++)
    for(i=0;i<N;i++)
      lambda[i+N*k]=0;
  
  /* on commence par lambda  en T (j=0...N-1) */
  
  phi_sc_etat(etat+N*T,&T,etatinf,etatsup,alpha,c,p, lambda+N*T,n_etat);
  
  for(k=T-1;k>0;k--)
    {
      L_sc_etat(cmdt+C*k,etat+N*k,&k,etatinf,etatsup,alpha,c,p,l,n_cmd,n_etat);
      f_sc_etat(cmdt+C*k,etat+N*k,&k,gf_etat,n_cmd,n_etat);
      
      for(i=0;i<N;i++)
	{
	  for(j=0;j<N;j++)
	    lambda[i+N*k]+=gf_etat[j+N*i]*lambda[j+N*(k+1)];
	  lambda[i+N*k]+=l[i];	  
	}
    }
  
  /*  Calcul du gradient du cout 
   *
   * on initialise le gradient a zero*/
  
  for(k=0;k<T;k++)
    for(i=0;i<C;i++)
      grd[i+k*C]=0;
  
  for(k=0;k<T;k++)
    {
      f_sc_cmd(cmdt+C*k,etat+N*k,&k,gf_cmd,n_cmd,n_etat);
      L_sc_cmd(cmdt+C*k,etat+N*k,&k,etatinf,etatsup,alpha,c,p,gC_cmd,n_cmd,n_etat);
      for(i=0;i<C;i++)
	{
	  for(j=0;j<N;j++)
	    grd[i+k*C]+=lambda[j+N*(k+1)]*gf_cmd[j+N*i];
	  
	  grd[i+k*C]+=gC_cmd[i];
	  
	}
    }
}




void THETA(double* cmdt, int* n_pas, int* n_cmd, int* n_etat, 
	   double* init_etat, double* etatinf, double* etatsup, 
	   double *alpha, double* etat,double* Th, int last)
{
  
  int T=*n_pas;  /* nombre de pas de temps */
  int N=*n_etat; /* taille de l'etat       */
  int C=*n_cmd;  /* taille de la commande  */

  /* les intermediaires */ 
  int i,k;
  
  /* enfin on dit quelle est la derniere variable Scilab utilisee*/
  BeginVar=8;

  /* on calcule l'etat(N x (T+1)) en fonction des  T commandes   
   */ 
  /*  on initialise l'etat */
  
  for(i=0;i<N;i++)
    etat[i]=init_etat[i];
    
  /* on calcule etat(t+1) avec etat(t)*/
  
  for(k=0;k<T;k++)
    f_sc(cmdt+C*k,etat+N*k,&k,etat+(k+1)*N,n_cmd,n_etat);

 
  /* on calcule le vecteur contrainte theta(u) */ 

  for(k=0;k<T+1;k++)
    for(i=0;i<N;i++)
      {
	Th[i+2*N*k]=(etat[i+N*k]-etatsup[i+N*k])/alpha[i];
	Th[i+N*(2*k+1)]=(etatinf[i+N*k]-etat[i+N*k])/alpha[i];
      }
}














