#include <stdio.h>
#include <stdlib.h>
#include "cout_dyn.h"
#include "stack-c.h"
#include "fonctions.h" 

/*  les noms des fonctions scilab sont dans les variables suivantes
---------------------------------------------------------------------*/

extern char phi_name[nlgh+1],phi_etat_name[nlgh+1];
extern char f_name[nlgh+1],f_cmd_name[nlgh+1],f_etat_name[nlgh+1];
extern char L_name[nlgh+1],L_cmd_name[nlgh+1],L_etat_name[nlgh+1];

/*-------------------------------------------------------------------
 *  compute the cost and its derivative with respect to u 
 * 
 *  J(cmd)= somme  (L(cmd(t),etat(t),t)  + phi(etat(T))
 *        t=0..T-1
 *
 *         etat(0)=e0
 *        etat(t+1)= F(cmd(t),etat(t),t)
 *----------------------------------------------------------------------
 */

FILE* fid;

void J_GRAD(double* cmdt,int* n_pas,int* n_cmd,int* n_etat,
	    double* init_etat,double* etat,double* grd,
	    double* FI,int last)
{
  /* les anciennes valeurs des tailles */ 
  static int o_lambda=0,o_gC_cmd=0,o_gf_cmd=0,o_gf_etat=0,o_l=0;
  
  /* les nouvelles */
  int T=*n_pas;  /* nombre de pas de temps */
  int N=*n_etat; /* taille de l'etat       */
  int C=*n_cmd;  /* taille de la commande  */

  static double* lambda;
  
  /* des intermediaires */
  double c; /*pour le calcul du cout*/
  static double* gf_etat=NULL;
  static double* gf_cmd=NULL;
  static double* gC_cmd=NULL;
  static double* l=NULL;
  int i,j,k;
  
  /* si c'est le dernier on nettoie */
  if(last==1)
    {
      net(&lambda);
      net(&gf_etat);
      net(&gf_cmd);
      net(&gC_cmd);
      net(&l);
      return ;
    }

  /* sinon on fait de la place */
  allocdouble(&lambda,N*(T+1),&o_lambda);
  allocdouble(&gf_etat,N*N,&o_gf_etat);
  allocdouble(&gf_cmd,N*C,&o_gf_cmd);
  allocdouble(&gC_cmd,C,&o_gC_cmd);
  allocdouble(&l,N,&o_l);

  /*-------------------------------DEBUT-------------------------------*/
  /* on calcul l'etat(N x (T+1)) en fonction des  T commandes   
   * ------------------------------------------------------------
   */
  /*  on initialise l'etat*/
  
  for(i=0;i<N;i++)
    etat[i]=init_etat[i];
  
  /* on calcule etat(t+1) avec etat(t)*/

  for(k=0;k<T;k++)
    {
      f(cmdt+C*k,etat+N*k,&k,etat+(k+1)*N,n_cmd,n_etat);
      
    }
  
  /* on en profite pour calculer le cout total*/
  /* on initialise les variables de sortie */
  
  *FI=0;
 
  /* calcul de J */
  
  for(k=0;k<T;k++)
    {
      L(cmdt+C*k,etat+N*k,&k,&c,n_cmd,n_etat);
      *FI+=c;
 
    }
  
  phi(etat+N*T,&T,&c,n_etat);
  *FI+=c;
 
  /*  on calcul l'etat adjoint (T x N+1)
   *  en fonction des T commandes et des T*(N+1) etats             
   */
  /* on initialise les lambdas */
  
  for(k=0;k<T+1;k++)
    for(i=0;i<N;i++)
      lambda[i+N*k]=0;
  
  /* on commence par lambda  en T (j=0...N-1) */
  
  phi_etat(etat+N*T,&T, lambda+N*T,n_etat);
  
  for(k=T-1;k>0;k--)
    {
      L_etat(cmdt+C*k,etat+N*k,&k,l,n_cmd,n_etat);
      f_etat(cmdt+C*k,etat+N*k,&k,gf_etat,n_cmd,n_etat);
      
      for(i=0;i<N;i++)
	{
	  for(j=0;j<N;j++)
	    lambda[i+N*k]+=gf_etat[j+N*i]*lambda[j+N*(k+1)];
	  lambda[i+N*k]+=l[i];	  
	}
    }
  
  /* Calcul du gradient du cout */
  /* on initialise le gradient a zero*/
  
  for(k=0;k<T;k++)
    for(i=0;i<C;i++)
      grd[i+k*C]=0;
  
  for(k=0;k<T;k++)
    {
      f_cmd(cmdt+C*k,etat+N*k,&k,gf_cmd,n_cmd,n_etat);
      L_cmd(cmdt+C*k,etat+N*k,&k,gC_cmd,n_cmd,n_etat);
      for(i=0;i<C;i++)
	{
	  for(j=0;j<N;j++)
	    grd[i+k*C]+=lambda[j+N*(k+1)]*gf_cmd[j+N*i];
	  
	  grd[i+k*C]+=gC_cmd[i];
	  
	}
    }
}

/**********************************************************************
 * instantaneous cost 
 **********************************************************************/

static void L(double* cmd,double* etat,int* temps,double* res,
       int* n_cmd,int* n_etat)
{
  int R=1,pass;
  /* on execute la macro L_name: resultat de taille 1 */ 
  pass=sciex3(L_name,cmd,etat,temps,res,n_cmd,n_etat,&R);
}
 
static void L_cmd(double* cmd,double* etat,int* temps,double* res,
	   int* n_cmd,int* n_etat)
{
  int pass;
  /* on execute la macro L_cmd_name: resultat de taille *n_cmd */ 
  pass=sciex3(L_cmd_name,cmd,etat,temps,res,n_cmd,n_etat,n_cmd);
}

static void L_etat(double* cmd,double* etat,int* temps,double* res,
	    int* n_cmd,int* n_etat)
{
  int pass;
  /* on execute la macro L_etat_name: resultat de taille *n_etat */ 
  pass=sciex3(L_etat_name,cmd,etat,temps,res,n_cmd,n_etat,n_etat);
}

/**********************************************************************
 * Final cost 
 **********************************************************************/

static void phi(double* etat,int* temps,double* res , int* n_etat)
{
  int R=1,pass;
  /* on execute la macro L_name: resultat de taille 1 */ 
  pass=sciex2(phi_name,etat,temps,res,n_etat,&R);
}

static void phi_etat(double* etat,int* temps,double* res , int* n_etat)
{
  int pass;
  /* on execute la macro L_etat_name: resultat de taille *n_etat */ 
  pass=sciex2(phi_etat_name,etat,temps,res,n_etat,n_etat);
}

/**********************************************************************
 * dynamics 
 **********************************************************************/

static void f(double* cmd,double* etat,int* temps,
       double* res,int* n_cmd,int* n_etat)

{
  int pass;
  /* on execute la macro L_etat_name: resultat de taille *n_etat */ 
  pass=sciex3(f_name,cmd,etat,temps,res,n_cmd,n_etat,n_etat);
}

static void f_etat(double* cmd,double* etat,int* temps,double* res,
	    int* n_cmd,int* n_etat)
{
  int pass,R=*n_etat*(*n_etat);
  /* on execute la macro L_etat_name: resultat de taille *n_etat*(*n_etat) */
  pass=sciex3(f_etat_name,cmd,etat,temps,res,n_cmd,n_etat,&R);
}

static void f_cmd(double* cmd,double* etat,int* temps,double* res,int* n_cmd,
	   int* n_etat)
{
  int pass, R=*n_etat*(*n_cmd);
  /* on execute la macro f_cmd_name: resultat de taille *n_etat*(*n_cmd) */
  pass=sciex3(f_cmd_name,cmd,etat,temps,res,n_cmd,n_etat,&R);
}

/*-------------------------------------------------------------------
 * utilities 
 *----------------------------------------------------------------------
 */

/* Allocate or reallocate memory */ 

static void allocdouble(double **r,int n_n,int *o_n )
{
  if (*o_n==0) 
    {
      *r=malloc(n_n*sizeof(double));
      *o_n=n_n;
    }
  else  
    {
      if ( *o_n < n_n ) 
	{	
	 *r=realloc(*r,n_n*sizeof(double));
	 *o_n=n_n;
	}
    }
}  

/* the associated free  */ 

static void net(double **r)
{
  if ((*r)!=NULL)
    free(*r);
  *r=NULL;
}






















