#include <string.h>
#include "stack-c.h"
#include "cout_dyn.h"
#include "doc_int.h"

/***************************************************
 * deal with errors in scilab functions 
 ***************************************************/

#include <setjmp.h>
static  jmp_buf Jcenv; 

#define MySciString(ibegin,name,mlhs,mrhs) \
  if( ! C2F(scistring)(ibegin,name,mlhs,mrhs,strlen(name))) \
       { longjmp(Jcenv,-1);return 0; } 

static int returned_from_longjump ;


/*********************************************************************
 * Interface for J_GRAD which computes the cost function and 
 * its derivatives 
 *                  [f,grf,lambda]=doc_J(cmd,e0)
 *********************************************************************/

int interfaceJ(char* fname)
{
  static int ierr;
  static int m1,n1,l1,m2,n2,l2; /* les variables d'entres*/
  static int m3,n3,l3,m4,n4,l4,m5,n5,l5;/* les outputs */
  static int minlhs=0, minrhs=2, maxlhs=3,maxrhs=2;
 
  CheckRhs(minrhs,maxrhs);
  CheckLhs(minlhs,maxlhs);

  GetRhsVar(1,"d",&m1,&n1,&l1); /* u(.) command */
  GetRhsVar(2,"d",&m2,&n2,&l2); /* x(0) */ 
 
  /*  n1 is the step size 
   *  m1 is the dimension of the control 
   *  m2 is the state dimension 
   */

  /* output variables */ 
  
  n3=1; m3=1;  CreateVar(3,"d",&m3,&n3,&l3);  
  n4=n1; m4=m1;  CreateVar(4,"d",&m4,&n4,&l4);
  
  /* l'etat est de dimension N*(T+1)*/
  n5=(n1+1); m5=m2; CreateVar(5,"d",&m5,&n5,&l5);

  if (( returned_from_longjump = setjmp(Jcenv)) != 0 )
    {
      Scierror(999,"%s: Internal error \r\n",fname);
      return 0;
    } 

  J_GRAD(stk(l1),&n1,&m1,&m2,stk(l2),stk(l5),stk(l4),stk(l3),0);
  
  LhsVar(1)=3;
  LhsVar(2)=4;
  LhsVar(3)=5;
  return 0;
}


/*********************************************************************
 *  interface for doc_setf which records the names of the macros 
 *  which code the current problem to be solved 
 *********************************************************************/

/* the current names are stored here */ 

char phi_name[nlgh+1],phi_etat_name[nlgh+1];
char f_name[nlgh+1],f_cmd_name[nlgh+1],f_etat_name[nlgh+1];
char L_name[nlgh+1],L_cmd_name[nlgh+1],L_etat_name[nlgh+1];

int interfaceSetf(char* fname)
{
  static int ierr;
  static int m1,n1,l1,m2,n2,l2,m3,n3,l3;/* les noms de L et de ses derivees  */
  static int m4,n4,l4,m5,n5,l5,m6,n6,l6;/* les noms de f et de ses derivees  */
  static int m7,n7,l7,m8,n8,l8;/* les noms de phi et de ses derivees*/
  static int minlhs=0, minrhs=8, maxlhs=1,maxrhs=8;

  CheckRhs(minrhs,maxrhs);
  CheckLhs(minlhs,maxlhs);

  /*          on recupere les variables entrees
  ------------------------------------------------*/
  GetRhsVar(1,"c",&m1,&n1,&l1); /* le nom de L        */
  GetRhsVar(2,"c",&m2,&n2,&l2); /* le nom de L_cmd    */
  GetRhsVar(3,"c",&m3,&n3,&l3); /* le nom de L_etat   */
  GetRhsVar(4,"c",&m4,&n4,&l4); /* le nom de f        */
  GetRhsVar(5,"c",&m5,&n5,&l5); /* le nom de f_cmd    */
  GetRhsVar(6,"c",&m6,&n6,&l6); /* le nom de f_etat   */
  GetRhsVar(7,"c",&m7,&n7,&l7); /* le nom de phi      */
  GetRhsVar(8,"c",&m8,&n8,&l8); /* le nom de phi_etat */
  
  strncpy(L_name,cstk(l1),nlgh);
  strncpy(L_cmd_name,cstk(l2),nlgh);
  strncpy(L_etat_name,cstk(l3),nlgh);
  strncpy(f_name,cstk(l4),nlgh);
  strncpy(f_cmd_name,cstk(l5),nlgh);
  strncpy(f_etat_name,cstk(l6),nlgh);
  strncpy(phi_name,cstk(l7),nlgh);
  strncpy(phi_etat_name,cstk(l8),nlgh);


  return 0;
}
  
/*********************************************************************
 * interface for doc_washup : the cleaning routine 
 *********************************************************************/

int interfacewashup(char* fname)
{
  
  static int ierr,un=1;
  static double p=1;
  static int minlhs=1, minrhs=0, maxlhs=1,maxrhs=0;

  CheckRhs(minrhs,maxrhs);
  CheckLhs(minlhs,maxlhs);

  if (( returned_from_longjump = setjmp(Jcenv)) != 0 )
    {
      Scierror(999,"%s: Internal error \r\n",fname);
      return 0;
    } 

  /* last parameter set to 1 for cleaning */ 

  J_GRAD(&p,&un,&un,&un,&p,&p,&p,&p,1); 
  return 0;
}

/*************************** interface sciex **************************/

int sciex3(char fct[nlgh+1],double* cmd,double* etat,int* temps,
	   double* res,int* n_cmd,int* n_etat, int* n_res)
{
  /* les variables utilisees sont les 6, 7 et 8 */
  static int l6,l7,l8;
  /* on utilise trois arguments et on ne renvoie qu'un resultat */
  static int ibegin=6,mlhs=1,mrhs=3;

  int i,un=1;
  
  CreateVar(6,"d",n_cmd,&un,&l6);
  for(i=0;i < *n_cmd;i++)
    stk(l6)[i]=cmd[i];
    
  CreateVar(7,"d",n_etat,&un,&l7);
  for(i=0;i < *n_etat;i++)
    stk(l7)[i]=etat[i];
    
  CreateVar(8,"d",&un,&un,&l8);
  stk(l8)[0]=*temps;
  
  /* execute la fonction fct */
  
  MySciString(&ibegin,fct,&mlhs,&mrhs);
 
  for(i=0;i<n_res[0];i++)
    res[i]=stk(l6)[i];
 
  return 0;
}


int sciex2(char fct[nlgh+1],double* etat,int* temps,double* res,
	   int* n_etat, int* n_res)
{
  /* les variables utilisees sont les 6, 7 */
  static int l6,l7;
  /* on utilise deux arguments et on ne renvoie qu'un resultat */
  static int ibegin=6,mlhs=1,mrhs=2;

  int i,un=1;
  
  CreateVar(6,"d",n_etat,&un,&l6);
  for(i=0;i < *n_etat;i++)
    stk(l6)[i]=etat[i];
    
  CreateVar(7,"d",&un,&un,&l7);
  stk(l7)[0]=*temps;

  /* execute la fonction fct */

  MySciString(&ibegin,fct,&mlhs,&mrhs);
  
  for(i=0;i<n_res[0];i++)
    res[i]=stk(l6)[i];
  
  return 0;
}








